#  Copyright (c) 2025.
#  Penetrum LLC (all rights reserved)
#  Copyright last updated: 4/22/25, 10:49 AM
#

def output(out, filename):
    raise NotImplementedError("PDF output is not implemented yet")
