#  Copyright (c) 2025.
#  Penetrum LLC (all rights reserved)
#  Copyright last updated: 5/1/25, 4:16 PM
#

import malcore_playbook.main.entry as entry


def run():
    """ CLI tool execution entrypoint """
    entry.main()
